/**
 * @class ValExt.Tooltip
 * Valence Portalにツールチップを追加
 */
export default class Tooltip {
  static owner={} ;
  constructor() {
    this.name = name;
    this.dataView = null;
    this.observer = null;
    this.observer_config = {
      childList: true, //対象ノードの子ノードに対する追加・削除の監視を有効に
      subtree: true, //対象ノードとその子孫ノードに対する変更の監視を有効に    
    };
    this.observerTarget = null;

    Ext.apply(Ext.tip.QuickTipManager.getQuickTip(), {
      maxWidth: 300,
      minWidth: 100,
      showDelay: 50      // Show 50ms after entering target
    });   
  }

  /**
   * @setter setOwner
   * @param {owner} 呼び出し元要素(ValExtオブジェクト)
  */
  setOwner(owner){
    Tooltip.owner = owner;
  } 

  /**
   * ToolTips追加対象監視のスタート  gValExt.Tooltip
  */

  /**
   * ToolTips追加対象監視のスタート
   * @method observeStart
   * @param {target} 監視対象のDOM要素
  */ 
  observeStart(target){
    if(! this.observer){
      this.observer = new MutationObserver(function(){
        Tooltip.owner.Tooltip.applyToolTips();  
      } );
      this.observer.observe(target, this.observer_config);
      this.observerTarget = target;
    }
  }

  /**
   * ToolTips要素を削除
   * @method removeAll
   */  
  removeAll(){
    if(!this.dataView) return ;
    for(var i =0 ; i < this.dataView.all.elements.length;i++){
      Ext.tip.QuickTipManager.unregister(this.dataView.all.elements[i]);
    }
  }

  /**
   * ToolTips追加実行済みであるかの確認
   * @method done
   * @returns {boolean} true Tooltip追加実行済み, false Tooltip未追加
   */   
  done(){
    if(!Ext.tip.QuickTipManager.tip) return false;
    let tipsCount =  Object.keys(Ext.tip.QuickTipManager.tip.targets).length ;
    let dataSourceCount  = this.dataView.dataSource.data.items.length;
    if(tipsCount == 0) return false; 
    if(tipsCount == dataSourceCount ) return true;
    if(tipsCount > dataSourceCount){
        this.removeAll();
        return false;
    }
  }

  /**
   * ToolTipsの適用
   * @method applyToolTips
   */  
  applyToolTips(){
    //ToolTips追加済みチェック
    if(this.done()) return ;
    //監視中断
    this.observer.disconnect();
    var loop = this.dataView.dataSource.data.items.length;
    for(var i=0; i< loop ; i++){
      Ext.tip.QuickTipManager.register({
        target:  this.dataView.all.elements[i],
        title: this.dataView.dataSource.data.items[i].data.name,
        text: this.dataView.dataSource.data.items[i].data.toolTip,
        width: 200,
        dismissDelay: 10000 // Hide after 10 seconds hover
      });
    }
    //監視再開
    this.observer.observe(this.observerTarget,this.observer_config);
  }
}